/*
   usr9105 : US Robotics 9105 router control system

   (c) 2005 (GPL) by Mark Wadham - mark.wadham@gmail.com

   Usage: ./usr9105 <ip> <user> <pass> <cmd>

      or: ./usr9105 -c <config file>

   See README for instructions.

   If you notice any bugs please email mark.wadham@gmail.com. Thanks.

   Hi Abbey!
*/

#include <stdio.h>
#include <netdb.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <netinet/in.h>

char output[20480];

int contains(char *data, char *str, int len)
{
	int i;
	if(len<1) return 0;
	if(strstr(data,str)!=0) return 1;
	return 0;
}

int get_socket(char *ip)
{
        struct sockaddr_in sin;
        int s;
                
        s = socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
                        
        sin.sin_addr.s_addr = inet_addr(ip);
        sin.sin_port    =       htons(23);
        sin.sin_family  =       AF_INET;
 
        if((connect(s,(struct sockaddr *)&sin,sizeof(sin))) <0) return -1;
	return s;
}

int authenticate(int s, char *user, char *pass)
{
	char packet[1024];
	int x=0;

	memset(packet,0,sizeof(packet));
	while((contains(packet,"Login:",x))==0)
	{
		memset(packet,0,sizeof(packet));
		x = recv(s,packet,sizeof(packet),0);
        }

	sprintf(packet,"%s\n",user);
	send(s,packet,strlen(packet),0);

        x=0; memset(packet,0,sizeof(packet));
        while((contains(packet,"Password:",x))==0)
        {
                memset(packet,0,sizeof(packet));
                x = recv(s,packet,sizeof(packet),0);
        }

	sprintf(packet,"%s\n",pass);
        send(s,packet,strlen(packet),0);
 
        while(1)
        {
                memset(packet,0,sizeof(packet));
                recv(s,packet,sizeof(packet),0);

		if(strstr(packet,"incorrect")!=0) return 0;
		if(strcmp(packet,"> ")==0) break;
        }

	return 1;
}

int send_command(int s, char *cmd)
{
	char packet[1024];
	char cmdbuf[1024];
	char buf[20480];
	unsigned int i,j;

	memset(output,0,sizeof(output));

	sprintf(cmdbuf,"%s\n",cmd);
	send(s,cmdbuf,strlen(cmdbuf),0);

	memset(packet,0,sizeof(packet));
	while(1)
	{
		memset(packet,0,sizeof(packet));
		recv(s,packet,sizeof(packet),0);
		strcat(output,packet);
		if(strcmp(packet,"> ")==0) break;
		if(packet[strlen(packet)-2]=='>' && packet[strlen(packet)-1]==' ') break;
	}

	memset(buf,0,sizeof(buf));
	for(i=0;i<strlen(output);i++)
		if(output[i]==10) break;
	for(j=i+1;j<strlen(output);j++)
		buf[j-i-1] = output[j];
	strcpy(output,buf);
	for(i=strlen(output)-1;i>0;i--)
		if(output[i]==10) break;
	output[i]=0;
}

int read_line(FILE *fp,char *buf,int maxlen)
{
	unsigned int c,x=0;
	memset(buf,0,maxlen);
	while((c=getc(fp))!=EOF)
	{
		if(c==10 || c==13) break;
		buf[x++] = c;
	}
	if(c==EOF) return 0;
	return 1;
}

int read_config(FILE *fp,char *ip,char *user,char *pass)
{
	char buf[256];

	if(!read_line(fp,(char *)&buf,256)) return 0;
	if(strncmp(buf,"ip=",3)!=0) return 0;
	strcpy(ip,(char *)&buf[3]);
	if(!read_line(fp,(char *)&buf,256)) return 0;
	if(strncmp(buf,"user=",5)!=0) return 0;
	strcpy(user,(char *)&buf[5]);
	if(!read_line(fp,(char *)&buf,256)) return 0;
	if(strncmp(buf,"pass=",5)!=0) return 0;
	strcpy(pass,(char *)&buf[5]);
}

void send_command_list(int s, FILE *fp)
{
	char linebuf[1024];
	int lb = 0;

	while(read_line(fp,(char *)&linebuf,1024))
	{
		if(strlen(linebuf)>0)
		{
			send_command(s,linebuf);
			printf("%s",output);
			if(strlen(output)>0) lb = 1;
		}
	}
	if(lb) printf("\n");
}

int main(int ac, char *av[])
{
	FILE *cfg;
	int fm = 0;
	char ip[16];
	char user[128];
	char pass[128];

	if(ac>2 && strcmp(av[1],"-c")==0)
	{
		if((cfg = fopen(av[2],"r"))==NULL)
		{
			printf("Error opening config file: %s\n",av[2]);
			return 0;
		}
		fm = 1;
		if(!read_config(cfg,(char *)&ip,(char *)&user,(char *)&pass))
		{
			printf("Error reading config file, please check syntax.\n");
			return 0;
		}
	}
	else if(ac<5)
	{
		printf("Usage: %s <ip> <user> <pass> <cmd>\n",av[0]);
		printf("   or: %s -c <cfgfile>\n",av[0]);
		return 0;
	}
	else
	{
		strcpy(ip,av[1]);
		strcpy(user,av[2]);
		strcpy(pass,av[3]);
	}

	int s = get_socket(ip);

	if(!authenticate(s,user,pass))
	{
		printf("Invalid access credentials.\n");
		return 0;
	}

	if(fm)	send_command_list(s,cfg);
	else
	{
		send_command(s,av[4]);
		printf("%s",output);
	}

	return 0;
}
